/*	Sample program for Olimex AVR-MT-128 with ATMEGA-128 processor
 *	Echoes back the received characters on the uart.
 *	Compile with AVRStudio+WinAVR (gcc version 3.4.6)
 */

#include "avr/io.h"
#include "iom128.h"

#define	__AVR_ATMEGA128__	1
#define OSCSPEED	16000000		/* in Hz */

void PORT_Init()
{
	PORTA = 0b00000000;		DDRA = 0b01000000;	//Relay set as output (Bit6 = 1)
	PORTB = 0b00000000;		DDRB = 0b00000000;
	PORTC = 0b00000000;		DDRC = 0b11110111;
	PORTD = 0b11000000;		DDRD = 0b00001000;	//TX set as output (Bit3 = 1)
	PORTE = 0b00000000;		DDRE = 0b00110000;	//Buzzer set as output (Bit4 = 1, Bit5 = 1)
	PORTF = 0b00000000;		DDRF = 0b00000000;
	PORTG = 0b00000000;		DDRG = 0b00000000;
}

void UART_Init(uint32_t Baud)
{
	unsigned int BaudRate = OSCSPEED / (16 * Baud) - 1;	/* as per pg. 173 of the user manual */

	//set BaudRate to registers UBRR1H and UBRR1L
	UBRR1H = (unsigned char) (BaudRate>>8);
	UBRR1L = (unsigned char) BaudRate;

	UCSR1B = UCSR1B | 0b00011000;	//enable Receiver and Transmitter (Bit3 = 1, Bit4 = 1)

	UCSR1C = UCSR1C | 0b10000110;	//Set frame format: 8 data (Bit1 = 1, Bit2 = 1), 1 stop bit (Bit3 = 0)
}

unsigned char UART_Receive()
{
	if (UCSR1A & 0b10000000)	//if there is unreaded data
		return UDR1;
	else						//no unreaded data
		return 0;
}

void UART_Transmit(unsigned char data)
{
	while (!(UCSR1A & 0b00100000));	//waiting until buffer is ready to receive

	UDR1 = data;
}

int main()
{
	unsigned char Ch;
	PORT_Init();
	UART_Init(9600);
	while (1)
	{
		Ch = UART_Receive();
		if (Ch)
		{
			UART_Transmit(Ch);
		}
	}
	return 0;
}
